@@ -7,7 +7,7 @@
 ##  Description:
 ##      see Daemon.pod file
 ################################################################################
-##  Copyright (C) 1997-2010 by Earl Hood and Detlef Pilzecker.
+##  Copyright (C) 1997-2011 by Earl Hood and Detlef Pilzecker.
 ##
 ##  All rights reserved.
 ##
@@ -21,7 +21,7 @@ package Proc::Daemon;
 use strict;
 use POSIX();
 
-$Proc::Daemon::VERSION = '0.05';
+$Proc::Daemon::VERSION = '0.06';
 
 
 ################################################################################
@@ -31,8 +31,8 @@ $Proc::Daemon::VERSION = '0.05';
 #   %Daemon_Settings are hash key=>values and can be:
 #     work_dir     => '/working/daemon/directory'   -> defaults to '/'
 #     child_STDIN  => '/path/to/daemon/STDIN.file'  -> defautls to '/dev/null'
-#     child_STDOUT => '/path/to/daemon/STDOUT.file' -> defaults to *\STDIN
-#     child_STDERR => '/path/to/daemon/STDERR.file' -> defaults to *\STDIN
+#     child_STDOUT => '/path/to/daemon/STDOUT.file' -> defaults to '/dev/null'
+#     child_STDERR => '/path/to/daemon/STDERR.file' -> defaults to '/dev/null'
 #     pid_file =>     '/path/to/pid/file.txt'       -> defaults to
 #       undef (= write no file)
 #     exec_command => 'perl /home/script.pl'        -> execute a system command
@@ -69,7 +69,8 @@ sub new {
 # Returns to the parent:
 #   - nothing (parent does exit) if the context is looking for no return value.
 #   - the PID(s) of the daemon(s) created.
-# Returns to the child : 0 | never returns if used with 'exec_command'.
+# Returns to the child (daemon):
+#   its PID (= 0) | never returns if used with 'exec_command'.
 ################################################################################
 sub Init {
     my Proc::Daemon $self = shift;
@@ -88,56 +89,105 @@ sub Init {
 
 
     # Open a filehandle to the anonymous temporary pid file.
-    open( MEMORY, "+>", undef ) || die "Can not open anonymous temporary pidfile: $!";
+    open( my $FH_MEMORY, "+>", undef ) || die "Can not open anonymous temporary pidfile: $!";
 
 
     # If system commands are to be executed, put them in a list.
     my @exec_command = ref( $self->{exec_command} ) eq 'ARRAY' ? @{ $self->{exec_command} } : ( $self->{exec_command} );
     $#exec_command = 0 if $#exec_command < 0;
 
-    # Run a daemon once or for every system command.
+    # Create a daemon for every system command.
     foreach my $exec_command ( @exec_command ) {
-        # First parent is running here.
+        # The first parent is running here.
 
 
-        # Using this subroutine or loop multiple times we must modify the files:
+        # Using this subroutine or loop multiple times we must modify the filenames:
         # 'child_STDIN', 'child_STDOUT', 'child_STDERR' and 'pid_file' for every
-        # daemon. A highter number will be appended to the files.
+        # daemon (a higher number will be appended to the filenames).
         $self->adjust_settings();
 
 
         # First fork.
         my $pid = Fork();
-        unless ( $pid || ! defined $pid ) {
-            # First child runs here.
+        if ( defined $pid && $pid == 0 ) {
+            # The first child runs here.
 
 
-            # Detach ourselves from the terminal.
+            # Set the new working directory.
+            chdir $self->{work_dir} or die "Can't <chdir> to $self->{work_dir}: $!";
+
+            # Clear the file creation mask.
+            umask 0;
+
+            # Detach the child from the terminal (no controlling tty), make it the
+            # session-leader and the process-group-leader of a new process group.
             die "Cannot detach from controlling terminal" if POSIX::setsid() < 0;
 
-            # Prevent possibility of acquiring a controling terminal.
-            local $SIG{'HUP'} = 'IGNORE';
+            # "Is ignoring SIGHUP necessary?
+            #
+            # It's often suggested that the SIGHUP signal should be ignored before
+            # the second fork to avoid premature termination of the process. The
+            # reason is that when the first child terminates, all processes, e.g.
+            # the second child, in the orphaned group will be sent a SIGHUP.
+            #
+            # 'However, as part of the session management system, there are exactly
+            # two cases where SIGHUP is sent on the death of a process:
+            #
+            #   1) When the process that dies is the session leader of a session that
+            #      is attached to a terminal device, SIGHUP is sent to all processes
+            #      in the foreground process group of that terminal device.
+            #   2) When the death of a process causes a process group to become
+            #      orphaned, and one or more processes in the orphaned group are
+            #      stopped, then SIGHUP and SIGCONT are sent to all members of the
+            #      orphaned group.' [2]
+            #
+            # The first case can be ignored since the child is guaranteed not to have
+            # a controlling terminal. The second case isn't so easy to dismiss.
+            # The process group is orphaned when the first child terminates and
+            # POSIX.1 requires that every STOPPED process in an orphaned process
+            # group be sent a SIGHUP signal followed by a SIGCONT signal. Since the
+            # second child is not STOPPED though, we can safely forego ignoring the
+            # SIGHUP signal. In any case, there are no ill-effects if it is ignored."
+            # Source: http://code.activestate.com/recipes/278731/
+            #
+           # local $SIG{'HUP'} = 'IGNORE';
 
             # Second fork.
+            # This second fork is not absolutely necessary, it is more a precaution.
+            # 1. Prevent possibility of reacquiring a controlling terminal.
+            # Without this fork the daemon would remain a session-leader. In
+            # this case there is a potential possibility that the process could
+            # reacquire a controlling terminal. E.g. if it opens a terminal device,
+            # without using the O_NOCTTY flag. In Perl this is normally the case
+            # when you use <open> on this kind of device, instead of <sysopen>
+            # with the O_NOCTTY flag set.
+            # Note: Because of the second fork the daemon will not be a session-
+            # leader and therefore Signals will not be send to other members of
+            # his process group. If you need the functionality of a session-leader
+            # you may want to call POSIX::setsid() manually on your daemon.
+            # 2. Detach the daemon completely from the parent.
+            # The double-fork prevents the daemon from becoming a zombie. It is
+            # needed in this module because the grandparent process can continue.
+            # Without the second fork and if a child exits before the parent
+            # and you forget to call <wait> in the parent you will get a zombie
+            # until the parent also terminates. Using the second fork we can be
+            # sure that the parent of the daemon is finished near by or before
+            # the daemon exits.
             $pid = Fork();
-            unless ( $pid || ! defined $pid ) {
-                # Here is the second child running.
+            if ( defined $pid && $pid == 0 ) {
+                # Here the second child is running.
 
 
-                # Set the new working directory.
-                chdir $self->{work_dir};
-
-                # Clear file creation mask.
-                umask 0;
-
                 # Close open file handles/descriptors.
-                # MEMORY also will be closed here.
+                # $FH_MEMORY also will be closed here.
                 foreach ( 0 .. OpenMax() ) { POSIX::close( $_ ) }
 
-                # Reopen STDIN, STDOUT and STDERR to '..._path' or write to /dev/null
-                open ( STDIN,  ( $self->{child_STDIN}  ? "+>$self->{child_STDIN}"  : "+>/dev/null" ) );
-                open ( STDOUT, ( $self->{child_STDOUT} ? "+>$self->{child_STDOUT}" : "+>&STDIN"    ) );
-                open ( STDERR, ( $self->{child_STDERR} ? "+>$self->{child_STDERR}" : "+>&STDIN"    ) );
+                # Reopen STDIN, STDOUT and STDERR to '..._path' or to /dev/null.
+                # Data written on a null special file is discarded. Reads from
+                # the null special file always return end of file.
+                open( STDIN,  "<",  $self->{child_STDIN}  || "/dev/null" );
+                open( STDOUT, "+>", $self->{child_STDOUT} || "/dev/null" );
+                open( STDERR, "+>", $self->{child_STDERR} || "/dev/null" );
 
 
                 # Execute a system command and never return.
@@ -150,27 +200,33 @@ sub Init {
                 }
 
 
-                # Return the childs own PID (0)
+                # Return the childs own PID (= 0)
                 return $pid;
             }
 
             # First child (= second parent) runs here.
 
+
             # Print the PID of the second child into ...
             # ... the anonymous temporary pid file.
             $pid ||= '';
-            print MEMORY "$pid\n";
-            close MEMORY;
+            print $FH_MEMORY "$pid\n";
+            close $FH_MEMORY;
 
             # ... the real 'pid_file'.
             if ( $self->{pid_file} ) {
-                open( PIDFILE, "+>", $self->{pid_file} ) || die "Can not open pidfile (pid_file => '$self->{pid_file}'): $!";
-                print PIDFILE $pid;
-                close PIDFILE;
+                open( my $FH_PIDFILE, "+>", $self->{pid_file} ) ||
+                    die "Can not open pidfile (pid_file => '$self->{pid_file}'): $!";
+                print $FH_PIDFILE $pid;
+                close $FH_PIDFILE;
             }
 
-            # Don't 'wait' for the second child to exit,
-            # even if we don't have a value in $exec_command
+
+            # Don't <wait> for the second child to exit,
+            # even if we don't have a value in $exec_command.
+            # The second child will become orphan by <exit> here, but then it
+            # will be adopted by init(8), which automatically performs a <wait>
+            # to remove the zombie when the child exits.
 
             exit;
         }
@@ -179,18 +235,19 @@ sub Init {
         # Only first parent runs here.
 
 
-        # Wait for the first child to exit!!
-        wait;
+        # A child that terminates, but has not been waited for becomes
+        # a zombie. So we wait for the first child to exit.
+        waitpid( $pid, 0 );
     }
 
 
     # Only first parent runs here.
 
 
-    # Get the second child/ren PIDs out of the anonymous temporary pid file.
-    seek( MEMORY, 0, 0 );
-    my @pid = map { chomp $_; $_ eq '' ? undef : $_ } <MEMORY>;
-    close MEMORY;
+    # Get the daemon PIDs out of the anonymous temporary pid file.
+    seek( $FH_MEMORY, 0, 0 );
+    my @pid = map { chomp $_; $_ eq '' ? undef : $_ } <$FH_MEMORY>;
+    close $FH_MEMORY;
 
 
     # Exit if the context is looking for no value (void context).
@@ -377,9 +434,9 @@ sub Kill_Daemon {
 
     if ( $killed && $pidfile ) {
         # Set PID in pid file to '0'.
-        if ( open( PIDFILE, "+>", $pidfile ) ) {
-            print PIDFILE '0';
-            close PIDFILE;
+        if ( open( my $FH_PIDFILE, "+>", $pidfile ) ) {
+            print $FH_PIDFILE '0';
+            close $FH_PIDFILE;
         }
         else { warn "Can not open pidfile (pid_file => '$pidfile'): $!" }
     }
@@ -412,9 +469,9 @@ sub get_pid {
             $pid = $string;
         }
         # Open the pidfile and get the PID from it.
-        elsif ( open( MEMORY, "<", $string ) ) {
-            $pid = <MEMORY>;
-            close MEMORY;
+        elsif ( open( my $FH_MEMORY, "<", $string ) ) {
+            $pid = <$FH_MEMORY>;
+            close $FH_MEMORY;
 
             die "I found no valid PID ('$pid') in the pidfile: '$string'" if $pid =~ /\D/s;
 
@@ -430,9 +487,9 @@ sub get_pid {
     # Try to get the PID out of the new() settings.
     if ( ! $pid ) {
         # Try to get the PID out of the 'pid_file' setting.
-        if ( $self->{pid_file} && open( MEMORY, "<", $self->{pid_file} ) ) {
-            $pid = <MEMORY>;
-            close MEMORY;
+        if ( $self->{pid_file} && open( my $FH_MEMORY, "<", $self->{pid_file} ) ) {
+            $pid = <$FH_MEMORY>;
+            close $FH_MEMORY;
 
             if ( ! $pid || ( $pid && $pid =~ /\D/s ) ) { $pid = undef }
             else { $pidfile = $self->{pid_file} }
